package algorithms;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import methods.Method;
import objects.Classifiable;
import objects.ClassifiableList;
import objects.Motion;
import objects.Template;

public class MethodEvaluatorClassification extends MethodEvaluator {

    private final int nFolds;

    private List<String> subjects;
    private List<Motion> motions;

    private List<List<Double>> scoresCCR;
    private List<List<List<Double>>> scoresListCMC;

    public MethodEvaluatorClassification(Method[] methods, File directoryAMC, int nFolds) {
        super(methods, directoryAMC);
        this.nFolds = nFolds;
    }
    
    public void setSubjects(List<String> subjects) {
        this.subjects = subjects;
    }

    public void setSubjects(int nClasses) {
        List<List<Classifiable>> classifiablesBySubject = new ClassifiableList(getClassifiables()).splitBySubject();
        int numberOfSubjects = classifiablesBySubject.size();
        if (nClasses < 2) {
            System.out.println("nClasses=" + nClasses + " < 2");
            return;
        }
        if (nClasses > numberOfSubjects) {
            System.out.println("nClasses=" + nClasses + " > " + "numberOfSubjects=" + numberOfSubjects);
            return;
        }
        Random random = new Random();
        List<String> subjectsEvaluation = new ArrayList();
        while (subjectsEvaluation.size() < nClasses) {
            String subject = classifiablesBySubject.get(random.nextInt(numberOfSubjects)).get(0).getSubject();
            if (!subjectsEvaluation.contains(subject)) {
                subjectsEvaluation.add(subject);
            }
        }
    }

    @Override
    public void initialize() {
        scoresCCR = new ArrayList();
        scoresListCMC = new ArrayList();
        for (int m = 0; m <= getNumberOfMethods(); m++) {
            scoresCCR.add(new ArrayList());
            scoresListCMC.add(new ArrayList());
            for (int rank = 0; rank <= subjects.size(); rank++) {
                scoresListCMC.get(m).add(new ArrayList());
            }
        }
    }

    @Override
    public void evaluate() throws IOException {
        for (int m = 0; m < getNumberOfMethods(); m++) {
            Method method = getMethod(m);
            motions = loadMotions(method, subjects);
            List<List<Motion>> motionsBySequence = new ClassifiableList(motions).splitBySequence();
            List<List<Motion>> folds = new ArrayList();
            for (int fFold = 0; fFold < nFolds; fFold++) {
                folds.add(new ArrayList());
            }
            for (int i = 0; i < motionsBySequence.size(); i++) {
                folds.get(i % nFolds).addAll(motionsBySequence.get(i));
            }
            for (int fFold = 0; fFold < nFolds; fFold++) {
//                System.out.println("fFold=" + fFold);
                List<Motion> motionsTest = folds.get(fFold);
                List<Motion> motionsLearning = new ArrayList();
                for (int f = 0; f < nFolds; f++) {
                    if (f != fFold) {
                        motionsLearning.addAll(folds.get(f));
                    }
                }
                method.learnClassifier(motionsLearning);
                setClassifier(method.getClassifier());
                List<Template> templatesTest = method.extractTemplates(motionsTest);
                List<Template> templatesGallery = method.extractTemplates(motionsLearning);
                List<Template> templatesAll = new ArrayList();
                templatesAll.addAll(templatesTest);
                templatesAll.addAll(templatesGallery);
                getClassifier().getDecision().setDistanceMatrix(calculateDistanceMatrix(templatesAll));
                double[] cumulativeMatchCharacteristic = getCumulativeMatchCharacteristic(templatesTest, templatesGallery);
                for (int rank = 0; rank < subjects.size(); rank++) {
                    scoresListCMC.get(m).get(rank).add(cumulativeMatchCharacteristic[rank]);
                }
                scoresCCR.get(m).add(cumulativeMatchCharacteristic[0]);
            }
        }
    }

    @Override
    public void print() {
        for (int m = 0; m < getNumberOfMethods(); m++) {
            System.out.print((m == 0 ? "" : ",") + getMethod(m).getName() + ",");
            System.out.print(toString(scoresCCR.get(m)));
            System.out.print("\r\n");
//            System.out.print("CMC\r\n");
//            for (int rank = 0; rank <= nClasses; rank++) {
//                System.out.print(toString(scoresListCMC.get(m).get(rank)));
//                System.out.print("\r\n");
//            }
        }
    }
}
