package algorithms;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import methods.Method;
import objects.Classifiable;
import objects.ClassifiableList;
import objects.DistanceMatrix;
import objects.Motion;
import objects.Template;

public class MethodEvaluatorClustering extends MethodEvaluator {

    private final QueryRange queryRange = new QueryRange();
    private final int fineness;

    private List<String> subjectsLearning;
    private List<String> subjectsEvaluation;
    private List<Motion> motionsLearning;
    private List<Motion> motionsEvaluation;

    private List<List<Double>> scoresDCT;
    private List<List<Double>> scoresTD;
    private List<List<Double>> scoresDBI;
    private List<List<Double>> scoresDI;
    private List<List<Double>> scoresSC;
    private List<List<Double>> scoresFDR;
    private List<List<Double>> scoresEER;
    private List<List<Double>> scoresAUC;
    private List<List<Double>> scoresMAP;
    private List<List<List<Double>>> scoresListFARFRR0;
    private List<List<List<Double>>> scoresListFARFRR1;
    private List<List<List<Double>>> scoresListTARFAR0;
    private List<List<List<Double>>> scoresListTARFAR1;
    private List<List<List<Double>>> scoresListRCLPCN0;
    private List<List<List<Double>>> scoresListRCLPCN1;
    private List<List<Double>> scoresPAHC;
    private List<List<Double>> scoresPKmeans;
    private List<List<Double>> scoresRIAHC;
    private List<List<Double>> scoresRIKmeans;
    private List<List<Double>> scoresFAHC;
    private List<List<Double>> scoresFKmeans;
    private List<List<Double>> scoresJIAHC;
    private List<List<Double>> scoresJIKmeans;
    private List<List<Double>> scoresFMIAHC;
    private List<List<Double>> scoresFMIKmeans;

    public MethodEvaluatorClustering(Method[] methods, File directoryAMC, int fineness) {
        super(methods, directoryAMC);
        this.fineness = fineness;
    }

    public void setSubjects(List<String> subjectsLearning, List<String> subjectsEvaluation) {
        this.subjectsLearning = subjectsLearning;
        this.subjectsEvaluation = subjectsEvaluation;
    }

    public void setSubjects(int nClassesLearning, int nClassesEvaluation) {
        int numberOfClasses = new ClassifiableList(getClassifiables()).splitBySubject().size();
        if (nClassesLearning < 2) {
            System.out.println("nClassesLearning=" + nClassesLearning + " < 2");
            return;
        }
        if (nClassesEvaluation < 2) {
            System.out.println("nClassesEvaluation=" + nClassesEvaluation + " < 2");
            return;
        }
        if (nClassesLearning + nClassesEvaluation > numberOfClasses) {
            System.out.println("nClassesLearning=" + nClassesLearning + " + " + "nClassesEvaluation=" + nClassesEvaluation + " > " + "numberOfClasses=" + numberOfClasses);
            return;
        }
        List<List<Classifiable>> classifiablesBySubject = new ClassifiableList(getClassifiables()).splitBySubject();
        int numberOfSubjects = classifiablesBySubject.size();
        Random random = new Random();
        subjectsLearning = new ArrayList();
        subjectsEvaluation = new ArrayList();
        while (subjectsLearning.size() < nClassesLearning) {
            String subject = classifiablesBySubject.get(random.nextInt(numberOfSubjects)).get(0).getSubject();
            if (!subjectsLearning.contains(subject)) {
                subjectsLearning.add(subject);
            }
        }
        while (subjectsEvaluation.size() < nClassesEvaluation) {
            String subject = classifiablesBySubject.get(random.nextInt(numberOfSubjects)).get(0).getSubject();
            if (!subjectsEvaluation.contains(subject) && !subjectsLearning.contains(subject)) {
                subjectsEvaluation.add(subject);
            }
        }
    }

    @Override
    public void initialize() {
        scoresDCT = new ArrayList();
        scoresTD = new ArrayList();
        scoresDBI = new ArrayList();
        scoresDI = new ArrayList();
        scoresSC = new ArrayList();
        scoresFDR = new ArrayList();
        scoresEER = new ArrayList();
        scoresAUC = new ArrayList();
        scoresMAP = new ArrayList();
        scoresListFARFRR0 = new ArrayList();
        scoresListFARFRR1 = new ArrayList();
        scoresListTARFAR0 = new ArrayList();
        scoresListTARFAR1 = new ArrayList();
        scoresListRCLPCN0 = new ArrayList();
        scoresListRCLPCN1 = new ArrayList();
        scoresPAHC = new ArrayList();
        scoresPKmeans = new ArrayList();
        scoresRIAHC = new ArrayList();
        scoresRIKmeans = new ArrayList();
        scoresFAHC = new ArrayList();
        scoresFKmeans = new ArrayList();
        scoresJIAHC = new ArrayList();
        scoresJIKmeans = new ArrayList();
        scoresFMIAHC = new ArrayList();
        scoresFMIKmeans = new ArrayList();
        for (int m = 0; m <= getNumberOfMethods(); m++) {
            scoresDCT.add(new ArrayList());
            scoresTD.add(new ArrayList());
            scoresDBI.add(new ArrayList());
            scoresDI.add(new ArrayList());
            scoresSC.add(new ArrayList());
            scoresFDR.add(new ArrayList());
            scoresEER.add(new ArrayList());
            scoresAUC.add(new ArrayList());
            scoresMAP.add(new ArrayList());
            scoresListFARFRR0.add(new ArrayList());
            scoresListFARFRR1.add(new ArrayList());
            scoresListTARFAR0.add(new ArrayList());
            scoresListTARFAR1.add(new ArrayList());
            scoresListRCLPCN0.add(new ArrayList());
            scoresListRCLPCN1.add(new ArrayList());
            scoresPAHC.add(new ArrayList());
            scoresPKmeans.add(new ArrayList());
            scoresRIAHC.add(new ArrayList());
            scoresRIKmeans.add(new ArrayList());
            scoresFAHC.add(new ArrayList());
            scoresFKmeans.add(new ArrayList());
            scoresJIAHC.add(new ArrayList());
            scoresJIKmeans.add(new ArrayList());
            scoresFMIAHC.add(new ArrayList());
            scoresFMIKmeans.add(new ArrayList());
            for (int f = 0; f <= fineness; f++) {
                scoresListFARFRR0.get(m).add(new ArrayList());
                scoresListFARFRR1.get(m).add(new ArrayList());
                scoresListTARFAR0.get(m).add(new ArrayList());
                scoresListTARFAR1.get(m).add(new ArrayList());
                scoresListRCLPCN0.get(m).add(new ArrayList());
                scoresListRCLPCN1.get(m).add(new ArrayList());
            }
        }
    }

    @Override
    public void evaluate() throws IOException {
        System.out.print("[" + subjectsLearning.size() + ";" + subjectsEvaluation.size() + "]," + subjectsLearning.toString().replaceAll(",", ";") + "," + subjectsEvaluation.toString().replaceAll(",", ";") + ", \r\n");//

        for (int m = 0; m < getNumberOfMethods(); m++) {
            Method method = getMethod(m);
            motionsLearning = loadMotions(method, subjectsLearning);
            motionsEvaluation = loadMotions(method, subjectsEvaluation);
            method.learnClassifier(motionsLearning);
            setClassifier(method.getClassifier());
            List<Template> templatesEvaluation = method.extractTemplates(motionsEvaluation);
            long beginning = System.currentTimeMillis();
            DistanceMatrix distanceMatrix = calculateDistanceMatrix(templatesEvaluation);
            long end = System.currentTimeMillis();

            System.out.print(method.getName() + ",");//

            double dct = (double) (end - beginning) / distanceMatrix.getSize();//
            scoresDCT.get(m).add(dct);//
            System.out.print(dct + ",");//

            double td = getTemplateDimensionality(templatesEvaluation);//
            scoresTD.get(m).add(td);//
            System.out.print(td + ",");//

            // INTERNAL EVALUATION
            queryRange.setDistanceMatrix(distanceMatrix);

            double dbi = getDaviesBouldinIndex(templatesEvaluation);//
            scoresDBI.get(m).add(dbi);//
            System.out.print(dbi + ",");//

            double di = getDunnIndex(templatesEvaluation);//
            scoresDI.get(m).add(di);//
            System.out.print(di + ",");//

            double sc = getSilhouetteCoefficient(templatesEvaluation);//
            scoresSC.get(m).add(sc);//
            System.out.print(sc + ",");//

            double fdr = getFishersDiscriminantRatio(templatesEvaluation);//
            scoresFDR.get(m).add(fdr);//
            System.out.print(fdr + ",");//

            double highRadiusFARFRR = 1;
            while (getFARFRR(highRadiusFARFRR, templatesEvaluation)[1] > 0) {
                highRadiusFARFRR *= 2;
            }
            double radiusEER = findRadiusEER(0, highRadiusFARFRR, templatesEvaluation);

            double eer = getFARFRR(radiusEER, templatesEvaluation)[0];//
            scoresEER.get(m).add(eer);//
            System.out.print(eer + ",");//

            for (int f = 0; f <= fineness; f++) {
                double radiusFARFRR = f * 2 * radiusEER / fineness;
                double[] ratesFARFRR = getFARFRR(radiusFARFRR, templatesEvaluation);
                scoresListFARFRR0.get(m).get(f).add(ratesFARFRR[0]);
                scoresListFARFRR1.get(m).get(f).add(ratesFARFRR[1]);
            }
            double highRadiusTARFAR = 1;
            while (getTARFAR(highRadiusTARFAR, templatesEvaluation)[0] < 1 || getTARFAR(highRadiusTARFAR, templatesEvaluation)[1] < 1) {
                highRadiusTARFAR *= 2;
            }
            double maxRadiusTARFAR = findMaxRadiusGARIAR(0, highRadiusTARFAR, templatesEvaluation);
            List<double[]> curveTARFAR = new ArrayList();
            List<double[]> gapsFAR = new ArrayList();
            double[] minRatesTARFAR = getTARFAR(0, templatesEvaluation);
            double[] maxRatesTARFAR = getTARFAR(maxRadiusTARFAR, templatesEvaluation);
            curveTARFAR.add(minRatesTARFAR);
            curveTARFAR.add(maxRatesTARFAR);
            gapsFAR.add(new double[]{minRatesTARFAR[1], maxRatesTARFAR[1], 0, maxRadiusTARFAR});
            for (int f = 1; f < fineness; f++) {
                double maxGapSize = 0;
                double radiusLow = 0;
                double radiusHigh = 0;
                double radiusTARFAR = 0;
                int maxGapIndex = 0;
                for (int gapIndex = 0; gapIndex < gapsFAR.size(); gapIndex++) {
                    double[] gap = gapsFAR.get(gapIndex);
                    double gapSize = gap[1] - gap[0];
                    if (maxGapSize < gapSize) {
                        maxGapSize = gapSize;
                        radiusLow = gap[2];
                        radiusHigh = gap[3];
                        radiusTARFAR = (radiusLow + radiusHigh) / 2;
                        maxGapIndex = gapIndex;
                    }
                }
                double[] maxGap = gapsFAR.get(maxGapIndex);
                double[] ratesTARFAR = getTARFAR(radiusTARFAR, templatesEvaluation);
                gapsFAR.remove(maxGap);
                gapsFAR.add(new double[]{maxGap[0], ratesTARFAR[1], radiusLow, radiusTARFAR});
                gapsFAR.add(new double[]{ratesTARFAR[1], maxGap[1], radiusTARFAR, radiusHigh});
                curveTARFAR.add(ratesTARFAR);
            }
            curveTARFAR = sortCurve(curveTARFAR, 1);
            for (int f = 0; f <= fineness; f++) {
                scoresListTARFAR0.get(m).get(f).add(curveTARFAR.get(f)[0]);
                scoresListTARFAR1.get(m).get(f).add(curveTARFAR.get(f)[1]);
            }

            double auc = getAreaUnderCurve(curveTARFAR, 1);//
            scoresAUC.get(m).add(auc);//
            System.out.print(auc + ",");//

            double highRadiusRCLPCN = 1;
            while (getRCLPCN(highRadiusRCLPCN, templatesEvaluation)[0] < 1) {
                highRadiusRCLPCN *= 2;
            }
            double maxRadiusRCLPCN = findMaxRadiusRCLPCN(0, highRadiusRCLPCN, templatesEvaluation);
            List<double[]> curveRCLPCN = new ArrayList();
            List<double[]> gapsRCL = new ArrayList();
            double[] minRatesRCLPCN = getRCLPCN(0, templatesEvaluation);
            double[] maxRatesRCLPCN = getRCLPCN(maxRadiusRCLPCN, templatesEvaluation);
            curveRCLPCN.add(minRatesRCLPCN);
            curveRCLPCN.add(maxRatesRCLPCN);
            gapsRCL.add(new double[]{minRatesRCLPCN[0], maxRatesRCLPCN[0], 0, maxRadiusRCLPCN});
            for (int f = 1; f < fineness; f++) {
                double maxGapSize = 0;
                double radiusLow = 0;
                double radiusHigh = 0;
                double radiusRCLPCN = 0;
                int maxGapIndex = 0;
                for (int gapIndex = 0; gapIndex < gapsRCL.size(); gapIndex++) {
                    double[] gap = gapsRCL.get(gapIndex);
                    double gapSize = gap[1] - gap[0];
                    if (maxGapSize < gapSize) {
                        maxGapSize = gapSize;
                        radiusLow = gap[2];
                        radiusHigh = gap[3];
                        radiusRCLPCN = (radiusLow + radiusHigh) / 2;
                        maxGapIndex = gapIndex;
                    }
                }
                double[] maxGap = gapsRCL.get(maxGapIndex);
                double[] ratesRCLPCN = getRCLPCN(radiusRCLPCN, templatesEvaluation);
                gapsRCL.remove(maxGap);
                gapsRCL.add(new double[]{maxGap[0], ratesRCLPCN[0], radiusLow, radiusRCLPCN});
                gapsRCL.add(new double[]{ratesRCLPCN[0], maxGap[1], radiusRCLPCN, radiusHigh});
                curveRCLPCN.add(ratesRCLPCN);
            }
            curveRCLPCN = sortCurve(curveRCLPCN, 0);
            for (int f = 0; f <= fineness; f++) {
                scoresListRCLPCN0.get(m).get(f).add(curveRCLPCN.get(f)[0]);
                scoresListRCLPCN1.get(m).get(f).add(curveRCLPCN.get(f)[1]);
            }

            double map = getAreaUnderCurve(curveRCLPCN, 0);//
            scoresMAP.get(m).add(map);//
            System.out.print(map + ",");//
            
            // EXTERNAL EVALUATION
            ClusteringKmeans kmeans = new ClusteringKmeans(subjectsEvaluation.size());
            kmeans.setDistanceMatrix(distanceMatrix);
            List<List<Template>> clustersKmeans = kmeans.cluster(templatesEvaluation);
            int[] contingencyKmeans = getContingency(clustersKmeans);
            int tpKmeans = contingencyKmeans[0];
            int fpKmeans = contingencyKmeans[1];
            int fnKmeans = contingencyKmeans[2];
            int tnKmeans = contingencyKmeans[3];

            ClusteringAgglomerativeHierarchical ahc = new ClusteringAgglomerativeHierarchical(subjectsEvaluation.size());
            ahc.setDistanceMatrix(distanceMatrix);
            List<List<Template>> clustersAHC = ahc.cluster(templatesEvaluation);
            int[] contingencyAHC = getContingency(clustersAHC);
            int tpAHC = contingencyAHC[0];
            int fpAHC = contingencyAHC[1];
            int fnAHC = contingencyAHC[2];
            int tnAHC = contingencyAHC[3];

            double pKmeans = getPurity(clustersKmeans);//
            scoresPKmeans.get(m).add(pKmeans);//
            System.out.print(pKmeans + ",");//

            double pAHC = getPurity(clustersAHC);//
            scoresPAHC.get(m).add(pAHC);//
            System.out.print(pAHC + ",");//

            double riKmeans = getRandIndex(tpKmeans, fpKmeans, fnKmeans, tnKmeans);//
            scoresRIKmeans.get(m).add(riKmeans);//
            System.out.print(riKmeans + ",");//

            double riAHC = getRandIndex(tpAHC, fpAHC, fnAHC, tnAHC);//
            scoresRIAHC.get(m).add(riAHC);//
            System.out.print(riAHC + ",");//

            double fKmeans = getFmeasure(tpKmeans, fpKmeans, fnKmeans, tnKmeans);//
            scoresFKmeans.get(m).add(fKmeans);//
            System.out.print(fKmeans + ",");//

            double fAHC = getFmeasure(tpAHC, fpAHC, fnAHC, tnAHC);//
            scoresFAHC.get(m).add(fAHC);//
            System.out.print(fAHC + ",");//

            double jiKmeans = getJaccardIndex(tpKmeans, fpKmeans, fnKmeans, tnKmeans);//
            scoresJIKmeans.get(m).add(jiKmeans);//
            System.out.print(jiKmeans + ",");//

            double jiAHC = getJaccardIndex(tpAHC, fpAHC, fnAHC, tnAHC);//
            scoresJIAHC.get(m).add(jiAHC);//
            System.out.print(jiAHC + ",");//

            double fmiKmeans = getFowlkesMallowsIndex(tpKmeans, fpKmeans, fnKmeans, tnKmeans);//
            scoresFMIKmeans.get(m).add(fmiKmeans);//
            System.out.print(fmiKmeans + ",");//

            double fmiAHC = getFowlkesMallowsIndex(tpAHC, fpAHC, fnAHC, tnAHC);//
            scoresFMIAHC.get(m).add(fmiAHC);//
            System.out.print(fmiAHC + "\r\n");//
        }
    }

    @Override
    public void print() {
        System.out.print("---\r\n");
        for (int m = 0; m < getNumberOfMethods(); m++) {
            System.out.print(getMethod(m).getName() + ",");
            System.out.print(toString(scoresDCT.get(m)));
            System.out.print(toString(scoresTD.get(m)));
            System.out.print(toString(scoresDBI.get(m)));
            System.out.print(toString(scoresDI.get(m)));
            System.out.print(toString(scoresSC.get(m)));
            System.out.print(toString(scoresFDR.get(m)));
            System.out.print(toString(scoresEER.get(m)));
            System.out.print(toString(scoresAUC.get(m)));
            System.out.print(toString(scoresMAP.get(m)));
            System.out.print(toString(scoresPKmeans.get(m)));
            System.out.print(toString(scoresPAHC.get(m)));
            System.out.print(toString(scoresRIKmeans.get(m)));
            System.out.print(toString(scoresRIAHC.get(m)));
            System.out.print(toString(scoresFKmeans.get(m)));
            System.out.print(toString(scoresFAHC.get(m)));
            System.out.print(toString(scoresJIKmeans.get(m)));
            System.out.print(toString(scoresJIAHC.get(m)));
            System.out.print(toString(scoresFMIKmeans.get(m)));
            System.out.print(toString(scoresFMIAHC.get(m)));
            System.out.print("\r\n");
//            System.out.print("FAR,FRR,TAR,FAR,RCL,PCN\r\n");
//            for (int f = 0; f <= fineness; f++) {
//                System.out.print(toString(scoresListFARFRR0.get(m).get(f)));
//                System.out.print(toString(scoresListFARFRR1.get(m).get(f)));
//                System.out.print(toString(scoresListTARFAR0.get(m).get(f)));
//                System.out.print(toString(scoresListTARFAR1.get(m).get(f)));
//                System.out.print(toString(scoresListRCLPCN0.get(m).get(f)));
//                System.out.print(toString(scoresListRCLPCN1.get(m).get(f)));
//                System.out.print("\r\n");
//            }
        }
    }

    private double[] getFARFRR(double radius, List<Template> templatesEvaluation) {
        double far = 0;
        double frr = 0;
        int countGenuine = 0;
        int countImpostor = 0;
        for (Template templateTest : templatesEvaluation) {
            String templateTestSubject = templateTest.getSubject();
            List<Template> templatesGallery = new ArrayList(templatesEvaluation);
            templatesGallery.remove(templateTest);
            List<Template> templatesGenuine = new ArrayList();
            List<Template> templatesImpostor = new ArrayList();
            for (Template templateGallery : templatesGallery) {
                if (templateGallery.getSubject().equals(templateTestSubject)) {
                    templatesGenuine.add(templateGallery);
                } else {
                    templatesImpostor.add(templateGallery);
                }
            }
            queryRange.setR(radius);
            List<Template> templatesRetrieved = queryRange.query(templateTest, templatesGallery);
            for (Template templateRetrieved : templatesRetrieved) {
                if (!templatesGenuine.contains(templateRetrieved)) {
                    far++;
                }
            }
            for (Template templateGenuine : templatesGenuine) {
                if (!templatesRetrieved.contains(templateGenuine)) {
                    frr++;
                }
            }
            countGenuine += templatesGenuine.size();
            countImpostor += templatesImpostor.size();
        }
        far /= countImpostor;
        frr /= countGenuine;
        return new double[]{far, frr};
    }

    private double findRadiusEER(double beg, double end, List<Template> templatesEvaluation) {
        double radiusRCLPCN = (end + beg) / 2;
        double[] ratesFARFRR = getFARFRR(radiusRCLPCN, templatesEvaluation);
        if (end - beg > 1) {
            if (ratesFARFRR[0] > ratesFARFRR[1]) {
                return findRadiusEER(beg, radiusRCLPCN, templatesEvaluation);
            } else {
                return findRadiusEER(radiusRCLPCN, end, templatesEvaluation);
            }
        }
        return radiusRCLPCN;
    }

    private double[] getTARFAR(double radius, List<Template> templatesEvaluation) {
        double gar = 0;
        double iar = 0;
        int countGenuine = 0;
        int countImpostor = 0;
        for (Template templateTest : templatesEvaluation) {
            String templateTestSubject = templateTest.getSubject();
            List<Template> templatesGallery = new ArrayList(templatesEvaluation);
            templatesGallery.remove(templateTest);
            List<Template> templatesGenuine = new ArrayList();
            List<Template> templatesImpostor = new ArrayList();
            for (Template templateGallery : templatesGallery) {
                if (templateGallery.getSubject().equals(templateTestSubject)) {
                    templatesGenuine.add(templateGallery);
                } else {
                    templatesImpostor.add(templateGallery);
                }
            }
            queryRange.setR(radius);
            List<Template> templatesRetrieved = queryRange.query(templateTest, templatesGallery);
            for (Template templateRetrieved : templatesRetrieved) {
                if (templatesGenuine.contains(templateRetrieved)) {
                    gar++;
                }
                if (templatesImpostor.contains(templateRetrieved)) {
                    iar++;
                }
            }
            countGenuine += templatesGenuine.size();
            countImpostor += templatesImpostor.size();
        }
        iar /= countImpostor;
        gar /= countGenuine;
        return new double[]{gar, iar};
    }

    private double findMaxRadiusGARIAR(double beg, double end, List<Template> templatesGallery) {
        double radiusGARIAR = (end + beg) / 2;
        double[] ratesGARIAR = getTARFAR(radiusGARIAR, templatesGallery);
        if (end - beg > 1) {
            if (ratesGARIAR[0] == 1 && ratesGARIAR[1] == 1) {
                return findMaxRadiusGARIAR(beg, radiusGARIAR, templatesGallery);
            } else {
                return findMaxRadiusGARIAR(radiusGARIAR, end, templatesGallery);
            }
        }
        return end;
    }

    private double getAreaUnderCurve(List<double[]> curve, int index) {
        double auc = 0;
        for (int l = 1; l < curve.size(); l++) {
            auc += 0.5 * Math.abs(curve.get(l)[index] - curve.get(l - 1)[index]) * Math.abs(curve.get(l)[1 - index] + curve.get(l - 1)[1 - index]);
        }
        return auc;
    }

    private double[] getRCLPCN(double radius, List<Template> templatesEvaluation) {
        double rcl = 0;
        double pcn = 0;
        int countRetrieved = 0;
        int countGenuine = 0;
        for (Template templateTest : templatesEvaluation) {
            String templateTestSubject = templateTest.getSubject();
            List<Template> templatesGallery = new ArrayList(templatesEvaluation);
            templatesGallery.remove(templateTest);
            List<Template> templatesGenuine = new ArrayList();
            for (Template templateGallery : templatesGallery) {
                if (templateGallery.getSubject().equals(templateTestSubject)) {
                    templatesGenuine.add(templateGallery);
                }
            }
            queryRange.setR(radius);
            List<Template> templatesRetrieved = queryRange.query(templateTest, templatesGallery);
            for (Template templateRetrieved : templatesRetrieved) {
                if (templatesGenuine.contains(templateRetrieved)) {
                    pcn++;
                }
            }
            for (Template templateGenuine : templatesGenuine) {
                if (templatesRetrieved.contains(templateGenuine)) {
                    rcl++;
                }
            }
            countRetrieved += templatesRetrieved.size();
            countGenuine += templatesGenuine.size();
        }
        pcn = countRetrieved == 0 ? 1 : pcn / countRetrieved;
        rcl /= countGenuine;
        return new double[]{rcl, pcn};
    }

    private double findMaxRadiusRCLPCN(double beg, double end, List<Template> templatesGallery) {
        double radiusRCLPCN = (end + beg) / 2;
        double[] ratesRCLPCN = getRCLPCN(radiusRCLPCN, templatesGallery);
        if (end - beg > 1) {
            if (ratesRCLPCN[0] == 1) {
                return findMaxRadiusRCLPCN(beg, radiusRCLPCN, templatesGallery);
            } else {
                return findMaxRadiusRCLPCN(radiusRCLPCN, end, templatesGallery);
            }
        }
        return end;
    }
}
