package objects;

import algorithms.DistanceTemplates;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DistanceMatrix {

    private final DistanceTemplates distanceTemplates;
    private final Map<String, Double> distanceMap = new HashMap();
    private final List<String> identifiers = new ArrayList();

    public DistanceMatrix(DistanceTemplates distanceTemplates) {
        this.distanceTemplates = distanceTemplates;
    }

    private String getKey(Template template1, Template template2) {
        return template1.getIdentifier() + "#" + template2.getIdentifier();
    }

    public void putDistance(Template template1, Template template2) {
        distanceMap.put(getKey(template1, template2), distanceTemplates.getDistance(template1, template2));
        String identifier = template1.getIdentifier();
        if (!identifiers.contains(identifier)) {
            identifiers.add(identifier);
        }
    }

    public double getDistance(Template template1, Template template2) {
        return distanceMap.get(getKey(template1, template2));
    }

    public int getSize() {
        return distanceMap.size();
    }

    public void print() {
        Collections.sort(identifiers);
        System.out.print(",");
        for (String identifier : identifiers) {
            System.out.print(identifier + ",");
        }
        System.out.print("\r\n");
        for (String identifier1 : identifiers) {
            System.out.print(identifier1 + ",");
            for (String identifier2 : identifiers) {
                System.out.print(distanceMap.get(identifier1 + "#" + identifier2) + ",");
            }
            System.out.print("\r\n");
        }
    }
}
